"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentConfigurationLoader = void 0;
const decorator_1 = require("../decorator");
const metadataManager_1 = require("../decorator/metadataManager");
const configService_1 = require("../service/configService");
const environmentService_1 = require("../service/environmentService");
const util = require("util");
const util_1 = require("../util");
const debug = util.debuglog('midway:debug');
class ComponentConfigurationLoader {
    constructor(container) {
        this.container = container;
        this.loadedMap = new WeakMap();
        this.namespaceList = [];
    }
    async load(module) {
        let namespace = decorator_1.MAIN_MODULE_KEY;
        // 可能导出多个
        const configurationExports = this.getConfigurationExport(module);
        if (!configurationExports.length)
            return;
        // 多个的情况，数据交给第一个保存
        for (let i = 0; i < configurationExports.length; i++) {
            const configurationExport = configurationExports[i];
            if (this.loadedMap.get(configurationExport)) {
                // 已经加载过就跳过循环
                continue;
            }
            let configurationOptions;
            // 函数式写法
            if (metadataManager_1.MetadataManager.hasOwnMetadata(decorator_1.CONFIGURATION_OBJECT_KEY, configurationExport)) {
                // 函数式写法
                configurationOptions = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.CONFIGURATION_OBJECT_KEY, configurationExport);
            }
            else {
                // 普通类写法
                configurationOptions = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.CONFIGURATION_KEY, configurationExport);
            }
            // 已加载标记，防止死循环
            this.loadedMap.set(configurationExport, true);
            if (configurationOptions) {
                if (configurationOptions.namespace !== undefined) {
                    namespace = configurationOptions.namespace;
                    this.namespaceList.push(namespace);
                }
                debug(`[core]: load configuration in namespace="${namespace}"`);
                this.addImports(configurationOptions.imports);
                this.addImportObjects(configurationOptions.importObjects);
                this.addImportConfigs(configurationOptions.importConfigs);
                this.addImportConfigFilter(configurationOptions.importConfigFilter);
                if (configurationOptions.detector) {
                    await configurationOptions.detector.run(this.container, namespace);
                }
                decorator_1.DecoratorManager.saveModule(decorator_1.CONFIGURATION_KEY, {
                    target: configurationExport,
                    namespace,
                });
            }
        }
        // bind module
        this.container.bindClass(module, {
            namespace,
        });
    }
    loadSync(module) {
        let namespace = decorator_1.MAIN_MODULE_KEY;
        // 可能导出多个
        const configurationExports = this.getConfigurationExport(module);
        if (!configurationExports.length)
            return;
        // 多个的情况，数据交给第一个保存
        for (let i = 0; i < configurationExports.length; i++) {
            const configurationExport = configurationExports[i];
            if (this.loadedMap.get(configurationExport)) {
                // 已经加载过就跳过循环
                continue;
            }
            let configurationOptions;
            // 函数式写法
            if (metadataManager_1.MetadataManager.hasOwnMetadata(decorator_1.CONFIGURATION_OBJECT_KEY, configurationExport)) {
                // 函数式写法
                configurationOptions = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.CONFIGURATION_OBJECT_KEY, configurationExport);
            }
            else {
                // 普通类写法
                configurationOptions = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.CONFIGURATION_KEY, configurationExport);
            }
            // 已加载标记，防止死循环
            this.loadedMap.set(configurationExport, true);
            if (configurationOptions) {
                if (configurationOptions.namespace !== undefined) {
                    namespace = configurationOptions.namespace;
                    this.namespaceList.push(namespace);
                }
                debug(`[core]: load configuration in namespace="${namespace}"`);
                this.addImports(configurationOptions.imports);
                this.addImportObjects(configurationOptions.importObjects);
                this.addImportConfigs(configurationOptions.importConfigs);
                this.addImportConfigFilter(configurationOptions.importConfigFilter);
                if (configurationOptions.detector) {
                    configurationOptions.detector.runSync(this.container, namespace);
                }
                decorator_1.DecoratorManager.saveModule(decorator_1.CONFIGURATION_KEY, {
                    target: configurationExport,
                    namespace,
                });
            }
        }
        // bind module
        this.container.bindClass(module, {
            namespace,
        });
    }
    addImportConfigs(importConfigs) {
        if (importConfigs) {
            if (Array.isArray(importConfigs)) {
                this.container.get(configService_1.MidwayConfigService).add(importConfigs);
            }
            else {
                this.container.get(configService_1.MidwayConfigService).addObject(importConfigs);
            }
        }
    }
    addImportConfigFilter(importConfigFilter) {
        if (importConfigFilter) {
            this.container.get(configService_1.MidwayConfigService).addFilter(importConfigFilter);
        }
    }
    addImports(imports = []) {
        // 处理 imports
        for (let importPackage of imports) {
            if (!importPackage)
                continue;
            if (typeof importPackage === 'string') {
                importPackage = require(importPackage);
            }
            if ('Configuration' in importPackage) {
                // component is object
                this.load(importPackage);
            }
            else if ('component' in importPackage) {
                if (importPackage?.enabledEnvironment) {
                    if (importPackage?.enabledEnvironment?.includes(this.container
                        .get(environmentService_1.MidwayEnvironmentService)
                        .getCurrentEnvironment())) {
                        this.load(importPackage.component);
                    }
                }
                else {
                    this.load(importPackage.component);
                }
            }
            else {
                this.load(importPackage);
            }
        }
    }
    /**
     * 注册 importObjects
     * @param objs configuration 中的 importObjects
     */
    addImportObjects(objs) {
        if (objs) {
            const keys = Object.keys(objs);
            for (const key of keys) {
                if (typeof objs[key] !== undefined) {
                    this.container.registerObject(key, objs[key]);
                }
            }
        }
    }
    getConfigurationExport(exports) {
        const mods = [];
        if ((0, util_1.isConfigurationExport)(exports)) {
            mods.push(exports);
        }
        else {
            for (const m in exports) {
                const module = exports[m];
                if ((0, util_1.isConfigurationExport)(module)) {
                    mods.push(module);
                }
            }
        }
        return mods;
    }
    getNamespaceList() {
        return this.namespaceList;
    }
}
exports.ComponentConfigurationLoader = ComponentConfigurationLoader;
//# sourceMappingURL=componentLoader.js.map