"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayContainer = void 0;
const decorator_1 = require("../decorator");
const util = require("util");
const definitionRegistry_1 = require("./definitionRegistry");
const interface_1 = require("../interface");
const constants_1 = require("../constants");
const objectDefinition_1 = require("../definitions/objectDefinition");
const functionDefinition_1 = require("../definitions/functionDefinition");
const managedResolverFactory_1 = require("./managedResolverFactory");
const events_1 = require("events");
const types_1 = require("../util/types");
const util_1 = require("../util");
const metadataManager_1 = require("../decorator/metadataManager");
const debug = util.debuglog('midway:debug');
const debugBind = util.debuglog('midway:bind');
const debugSpaceLength = 9;
class MidwayContainer {
    get id() {
        return this._id;
    }
    constructor() {
        this._resolverFactory = null;
        this._registry = null;
        this._identifierMapping = null;
        // 仅仅用于兼容 requestContainer 的 ctx
        this.ctx = constants_1.SINGLETON_CONTAINER_CTX;
        this.attrMap = new Map();
        this.namespaceSet = new Set();
        this._id = util_1.Utils.generateRandomId();
        // 防止直接从applicationContext.getAsync or get对象实例时依赖当前上下文信息出错
        // ctx is in requestContainer
        this.registerObject(constants_1.REQUEST_CTX_KEY, this.ctx);
    }
    get objectCreateEventTarget() {
        if (!this._objectCreateEventTarget) {
            this._objectCreateEventTarget = new events_1.EventEmitter();
        }
        return this._objectCreateEventTarget;
    }
    get registry() {
        if (!this._registry) {
            this._registry = new definitionRegistry_1.ObjectDefinitionRegistry();
        }
        return this._registry;
    }
    set registry(registry) {
        this._registry = registry;
    }
    get identifierMapping() {
        if (!this._identifierMapping) {
            this._identifierMapping = this.registry.getIdentifierRelation();
        }
        return this._identifierMapping;
    }
    addNamespace(ns) {
        this.namespaceSet.add(ns);
    }
    bindClass(exports, options) {
        if (types_1.Types.isClass(exports) || types_1.Types.isFunction(exports)) {
            this.bindModule(exports, options);
        }
        else {
            for (const m in exports) {
                const module = exports[m];
                if (types_1.Types.isClass(module) || types_1.Types.isFunction(module)) {
                    this.bindModule(module, options);
                }
            }
        }
    }
    bind(identifier, target, options) {
        if (types_1.Types.isClass(identifier) || types_1.Types.isFunction(identifier)) {
            return this.bindModule(identifier, target);
        }
        if (this.registry.hasDefinition(identifier)) {
            // 如果 definition 存在就不再重复 bind
            return;
        }
        if (options?.bindHook) {
            options.bindHook(target, options);
        }
        let definition;
        if (types_1.Types.isClass(target)) {
            definition = new objectDefinition_1.ObjectDefinition();
            definition.name = decorator_1.DecoratorManager.getProviderName(target);
        }
        else {
            definition = new functionDefinition_1.FunctionDefinition();
            if (!types_1.Types.isAsyncFunction(target)) {
                definition.asynchronous = false;
            }
            definition.name = definition.id;
        }
        definition.path = target;
        definition.id = identifier;
        definition.srcPath = options?.srcPath || null;
        definition.namespace = options?.namespace || '';
        definition.scope = options?.scope || interface_1.ScopeEnum.Request;
        definition.createFrom = options?.createFrom;
        if (definition.srcPath) {
            debug(`[core]: bind id "${definition.name} (${definition.srcPath}) ${identifier}"`);
        }
        else {
            debug(`[core]: bind id "${definition.name}" ${identifier}`);
        }
        // inject constructor
        const constructorProps = metadataManager_1.MetadataManager.getPropertiesWithMetadata(decorator_1.CONSTRUCTOR_INJECT_KEY, target);
        if (constructorProps['default']) {
            const argsLen = constructorProps['default'].length;
            const constructorArgs = new Array(argsLen);
            for (const constructorMeta of constructorProps['default']) {
                debugBind(`${' '.repeat(debugSpaceLength)}inject constructor => [${JSON.stringify(constructorMeta)}]`);
                constructorArgs[constructorMeta.parameterIndex] = constructorMeta;
            }
            definition.constructorArgs = constructorArgs;
        }
        // inject properties
        const props = metadataManager_1.MetadataManager.getPropertiesWithMetadata(decorator_1.PROPERTY_INJECT_KEY, target);
        for (const p in props) {
            const propertyMeta = props[p];
            debugBind(`${' '.repeat(debugSpaceLength)}inject properties => [${JSON.stringify(propertyMeta)}]`);
            definition.properties.set(propertyMeta.targetKey, propertyMeta);
        }
        // inject custom properties
        const customProps = metadataManager_1.MetadataManager.getPropertiesWithMetadata(decorator_1.CUSTOM_PROPERTY_INJECT_KEY, target);
        for (const p in customProps) {
            for (const propertyMeta of customProps[p]) {
                definition.handlerProps.push(propertyMeta);
            }
        }
        // @async, @init, @destroy
        const objDefMetadata = metadataManager_1.MetadataManager.getPropertiesWithMetadata(decorator_1.OBJECT_DEFINITION_KEY, target);
        const objDefOptions = {};
        for (const p in objDefMetadata) {
            Object.assign(objDefOptions, objDefMetadata[p][0]);
        }
        if (objDefOptions.initMethod) {
            debugBind(`${' '.repeat(debugSpaceLength)}register initMethod = ${objDefOptions.initMethod}`);
            definition.initMethod = objDefOptions.initMethod;
        }
        if (objDefOptions.destroyMethod) {
            debugBind(`${' '.repeat(debugSpaceLength)}register destroyMethod = ${objDefOptions.destroyMethod}`);
            definition.destroyMethod = objDefOptions.destroyMethod;
        }
        const scopeMetadata = metadataManager_1.MetadataManager.getOwnMetadata(decorator_1.SCOPE_KEY, target);
        if (scopeMetadata) {
            const { scope, allowDowngrade } = scopeMetadata;
            debugBind(`${' '.repeat(debugSpaceLength)}register scope = ${scope}`);
            definition.scope = scope;
            debugBind(`${' '.repeat(debugSpaceLength)}register allowDowngrade = ${allowDowngrade}`);
            definition.allowDowngrade = allowDowngrade;
        }
        this.objectCreateEventTarget.emit(interface_1.ObjectLifeCycleEvent.BEFORE_BIND, target, {
            context: this,
            definition,
            replaceCallback: newDefinition => {
                definition = newDefinition;
            },
        });
        if (definition) {
            this.registry.registerDefinition(definition.id, definition);
        }
    }
    bindModule(module, options) {
        if (types_1.Types.isClass(module)) {
            const providerId = decorator_1.DecoratorManager.getProviderUUId(module);
            if (providerId) {
                this.identifierMapping.saveClassRelation(module, options?.namespace);
                this.bind(providerId, module, options);
            }
            else {
                // no provide or js class must be skip
            }
        }
        else {
            const info = module[constants_1.FUNCTION_INJECT_KEY];
            if (info && info.id) {
                if (!info.scope) {
                    info.scope = interface_1.ScopeEnum.Request;
                }
                const uuid = util_1.Utils.generateRandomId();
                this.identifierMapping.saveFunctionRelation(info.id, uuid);
                this.bind(uuid, module, {
                    scope: info.scope,
                    namespace: options.namespace,
                    srcPath: options.srcPath,
                    createFrom: options.createFrom,
                });
            }
        }
    }
    setAttr(key, value) {
        this.attrMap.set(key, value);
    }
    getAttr(key) {
        return this.attrMap.get(key);
    }
    getIdentifier(identifier) {
        if (typeof identifier !== 'string') {
            identifier = decorator_1.DecoratorManager.getProviderUUId(identifier);
        }
        return identifier;
    }
    getManagedResolverFactory() {
        if (!this._resolverFactory) {
            this._resolverFactory = new managedResolverFactory_1.ManagedResolverFactory(this);
        }
        return this._resolverFactory;
    }
    async stop() {
        await this.getManagedResolverFactory().destroyCache();
        this.registry.clearAll();
        this.attrMap.clear();
    }
    // ready() {
    //   return this.loadDefinitions();
    // }
    get(identifier, args) {
        return this.getManagedResolverFactory().create(identifier, args, this);
    }
    async getAsync(identifier, args) {
        return this.getManagedResolverFactory().createAsync(identifier, args, this);
    }
    /**
     * proxy registry.registerObject
     * @param {ObjectIdentifier} identifier
     * @param target
     */
    registerObject(identifier, target) {
        this.registry.registerObject(identifier, target);
    }
    onBeforeBind(fn) {
        this.objectCreateEventTarget.on(interface_1.ObjectLifeCycleEvent.BEFORE_BIND, fn);
    }
    onBeforeObjectCreated(fn) {
        this.objectCreateEventTarget.on(interface_1.ObjectLifeCycleEvent.BEFORE_CREATED, fn);
    }
    onObjectCreated(fn) {
        this.objectCreateEventTarget.on(interface_1.ObjectLifeCycleEvent.AFTER_CREATED, fn);
    }
    onObjectInit(fn) {
        this.objectCreateEventTarget.on(interface_1.ObjectLifeCycleEvent.AFTER_INIT, fn);
    }
    onBeforeObjectDestroy(fn) {
        this.objectCreateEventTarget.on(interface_1.ObjectLifeCycleEvent.BEFORE_DESTROY, fn);
    }
    hasNamespace(ns) {
        return this.namespaceSet.has(ns);
    }
    getNamespaceList() {
        return Array.from(this.namespaceSet);
    }
    hasDefinition(identifier) {
        return this.registry.hasDefinition(identifier);
    }
    getDefinition(identifier) {
        return this.registry.getDefinition(identifier);
    }
    hasObject(identifier) {
        return this.registry.hasObject(identifier);
    }
    getObject(identifier) {
        return this.registry.getObject(identifier);
    }
    getInstanceScope(instance) {
        if (instance[constants_1.CONTAINER_OBJ_SCOPE]) {
            return instance[constants_1.CONTAINER_OBJ_SCOPE];
        }
        return undefined;
    }
}
exports.MidwayContainer = MidwayContainer;
//# sourceMappingURL=container.js.map