"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManagedResolverFactory = void 0;
/**
 * 管理对象解析构建
 */
const constants_1 = require("../constants");
const interface_1 = require("../interface");
const util = require("util");
const error_1 = require("../error");
const functionDefinition_1 = require("../definitions/functionDefinition");
const debug = util.debuglog('midway:debug');
function createProxy(factory) {
    let instance;
    return new Proxy({}, {
        get(target, prop) {
            if (!instance) {
                instance = factory();
            }
            return instance[prop];
        },
    });
}
function formatObjectIdentifier(identifier) {
    if (typeof identifier === 'function') {
        return identifier();
    }
    return identifier;
}
/**
 * 解析工厂
 */
class ManagedResolverFactory {
    constructor(context) {
        this.creating = new Map();
        this.singletonCache = new Map();
        this.context = context;
    }
    /**
     * 同步创建对象
     * @param identifier
     * @param args
     * @param currentContext
     */
    create(identifier, args = [], currentContext) {
        const name = typeof identifier === 'string' ? identifier : identifier.name;
        identifier = currentContext.getIdentifier(identifier);
        debug('[core]: create "%s(%s)"', identifier, name);
        if (currentContext.hasObject(identifier)) {
            return currentContext.getObject(identifier);
        }
        const definition = this.getObjectDefinition(identifier);
        if (!definition &&
            currentContext !== this.context &&
            this.context.hasObject(identifier)) {
            return this.context.getObject(identifier);
        }
        if (!definition) {
            throw new error_1.MidwayDefinitionNotFoundError(identifier, name);
        }
        const pendingInitQueue = [];
        const pendingObjectCache = new Map();
        const instance = this.createInstance(identifier, name, args, definition, false, false, currentContext, pendingObjectCache, pendingInitQueue);
        pendingObjectCache.clear();
        const newInstance = this.initializeInstance(instance, definition, pendingInitQueue);
        return newInstance ?? instance;
    }
    async createAsync(identifier, args = [], currentContext) {
        const name = typeof identifier === 'string' ? identifier : identifier.name;
        identifier = currentContext.getIdentifier(identifier);
        debug('[core]: createAsync "%s(%s)"', identifier, name);
        if (currentContext.hasObject(identifier)) {
            return currentContext.getObject(identifier);
        }
        const definition = this.getObjectDefinition(identifier);
        if (!definition &&
            currentContext !== this.context &&
            this.context.hasObject(identifier)) {
            return this.context.getObject(identifier);
        }
        if (!definition) {
            throw new error_1.MidwayDefinitionNotFoundError(identifier, name);
        }
        const pendingInitQueue = [];
        const pendingObjectCache = new Map();
        const instance = this.createInstance(identifier, definition?.name ?? name, args, definition, true, false, currentContext, pendingObjectCache, pendingInitQueue);
        pendingObjectCache.clear();
        const newInstance = await this.initializeInstanceAsync(instance, definition, pendingInitQueue);
        return newInstance ?? instance;
    }
    async destroyCache() {
        for (const key of this.singletonCache.keys()) {
            const definition = this.getObjectDefinition(key);
            if (definition.creator) {
                const inst = this.singletonCache.get(key);
                this.getObjectEventTarget().emit(interface_1.ObjectLifeCycleEvent.BEFORE_DESTROY, inst, {
                    context: this.context,
                    definition,
                });
                await definition.creator.doDestroyAsync(inst);
            }
        }
        this.singletonCache.clear();
        this.creating.clear();
    }
    getObjectEventTarget() {
        return this.context.objectCreateEventTarget;
    }
    checkSingletonInvokeRequest(definition, key, currentContext) {
        if (definition.isSingletonScope()) {
            const managedRef = definition.properties.get(key);
            if (currentContext.hasDefinition(managedRef?.id)) {
                const propertyDefinition = currentContext.getDefinition(managedRef.id);
                if (propertyDefinition.isRequestScope() &&
                    !propertyDefinition.allowDowngrade) {
                    throw new error_1.MidwaySingletonInjectRequestError(definition.path.name, propertyDefinition.path.name);
                }
            }
        }
        return true;
    }
    setInstanceScope(inst, scope) {
        if (typeof inst === 'object') {
            if (scope === interface_1.ScopeEnum.Request &&
                inst[constants_1.REQUEST_OBJ_CTX_KEY] === constants_1.SINGLETON_CONTAINER_CTX) {
                scope = interface_1.ScopeEnum.Singleton;
            }
            Object.defineProperty(inst, constants_1.CONTAINER_OBJ_SCOPE, {
                value: scope,
                writable: false,
                enumerable: false,
                configurable: false,
            });
        }
    }
    createInstance(identifier, name, args = [], definition, isAsync, isLazyInject, currentContext, pendingObjectCache, pendingInitQueue, creationPath = new Set(), replaceCallback) {
        identifier = currentContext.getIdentifier(identifier);
        if (currentContext.hasObject(identifier)) {
            return currentContext.getObject(identifier);
        }
        definition = definition ?? currentContext.getDefinition(identifier);
        if (!definition &&
            currentContext !== this.context &&
            this.context.hasObject(identifier)) {
            return this.context.getObject(identifier);
        }
        if (!definition) {
            throw new error_1.MidwayDefinitionNotFoundError(identifier, name, this.translateIdentifiers(Array.from(creationPath)));
        }
        if (definition.isSingletonScope()) {
            currentContext = this.context;
            if (this.singletonCache.has(definition.id)) {
                debug(`[core]: "${definition.id}(${definition.name})" get from singleton cache.`);
                return this.singletonCache.get(definition.id);
            }
        }
        // 使用 creationPath 检查循环依赖
        if (creationPath.has(definition.id)) {
            if (isLazyInject) {
                return createProxy(() => {
                    return currentContext.get(definition.id);
                });
            }
            else {
                const cycle = Array.from(creationPath).concat(definition.id);
                throw new error_1.MidwayCommonError(`Circular dependency detected: ${this.translateIdentifiers(cycle).join(' -> ')}`);
            }
        }
        if (pendingObjectCache.has(definition.id)) {
            return pendingObjectCache.get(definition.id);
        }
        creationPath.add(definition.id);
        // Pre-initialize dependencies
        if (definition.hasDependsOn()) {
            for (const dep of definition.dependsOn) {
                debug('[core]: id = %s init depend %s.', definition.id, dep);
                this.createInstance(dep, dep, [], undefined, isAsync, false, currentContext, pendingObjectCache, pendingInitQueue, new Set(creationPath));
            }
        }
        // Get class or function from definition
        const Clzz = definition.creator.load();
        // Get constructor args
        let constructorArgs = new Array(definition.constructorArgs.length);
        if (args && Array.isArray(args) && args.length > 0) {
            constructorArgs = args;
        }
        else {
            // init constructor args
            for (let i = 0; i < definition.constructorArgs.length; i++) {
                const arg = definition.constructorArgs[i];
                if (arg === undefined)
                    continue;
                arg.id = formatObjectIdentifier(arg.id);
                debug('[core]: constructor arg "%s(%s)", pos=%s, in "%s".', arg.id, arg.name, arg.parameterIndex, name);
                constructorArgs[i] = this.createInstance(arg.id, arg.name, [], undefined, isAsync, arg.isLazyInject, currentContext, pendingObjectCache, pendingInitQueue, new Set(creationPath));
            }
        }
        // Emit before created event
        this.getObjectEventTarget().emit(interface_1.ObjectLifeCycleEvent.BEFORE_CREATED, Clzz, {
            constructorArgs,
            context: currentContext,
        });
        // Create instance
        let inst = definition.creator.doConstruct(Clzz, constructorArgs);
        if (!inst) {
            throw new error_1.MidwayCommonError(`${definition.id} construct return undefined`);
        }
        // Binding ctx object
        if (definition.isRequestScope() &&
            definition.constructor.name === 'ObjectDefinition') {
            debug('[core]: "%s(%s)" inject ctx', definition.id, definition.name);
            // set related ctx
            Object.defineProperty(inst, constants_1.REQUEST_OBJ_CTX_KEY, {
                value: currentContext.get(constants_1.REQUEST_CTX_KEY),
                writable: false,
                enumerable: false,
            });
        }
        pendingObjectCache.set(definition.id, inst);
        // Set properties
        if (definition.properties) {
            const keys = Array.from(definition.properties.keys());
            for (const key of keys) {
                this.checkSingletonInvokeRequest(definition, key, currentContext);
                const resolver = definition.properties.get(key);
                resolver.id = formatObjectIdentifier(resolver.id);
                // if (
                //   resolver.injectMode === InjectModeEnum.Class &&
                //   !(this.getRootContext(currentContext)).hasDefinition(
                //     resolver.id
                //   )
                // ) {
                //   if (resolver.name === 'loggerService') {
                //     throw new MidwayInconsistentVersionError();
                //   } else {
                //     throw new MidwayMissingImportComponentError(resolver.name);
                //   }
                // }
                debug('[core]: property "%s(%s)", in "%s".', resolver.id || resolver.name, resolver.name, name);
                inst[key] = this.createInstance(resolver.id || resolver.name, resolver.name, resolver.args, undefined, isAsync, resolver.isLazyInject, currentContext, pendingObjectCache, pendingInitQueue, new Set(creationPath), newValue => {
                    inst[key] = newValue;
                });
            }
        }
        this.getObjectEventTarget().emit(interface_1.ObjectLifeCycleEvent.AFTER_CREATED, inst, {
            context: currentContext,
            definition,
            replaceCallback: ins => {
                inst = ins;
            },
        });
        debug('[core]: put "%s(%s)" to pending init queue.', definition.id, definition.name);
        // Set init function to pending init queue
        if (isAsync) {
            pendingInitQueue.push([
                inst,
                definition,
                () => definition.creator.doInitAsync(inst, currentContext),
                replaceCallback,
            ]);
        }
        else {
            pendingInitQueue.push([
                inst,
                definition,
                () => definition.creator.doInit(inst, currentContext),
                replaceCallback,
            ]);
        }
        return inst;
    }
    initializeInstance(instance, targetDefinition, pendingInitQueue) {
        const initializingSet = new Map();
        const initializedInstances = new Map();
        for (const [obj, definition, initFunc, replaceCallback,] of pendingInitQueue) {
            if (!initializedInstances.has(definition.id)) {
                initializingSet.set(definition.id, obj);
                debug('[core]: load "%s(%s)" from pending init queue and ready to init.', definition.id, definition.name);
                const res = initFunc?.();
                if (definition instanceof functionDefinition_1.FunctionDefinition) {
                    initializedInstances.set(definition.id, res);
                    replaceCallback?.(res);
                }
                else {
                    initializedInstances.set(definition.id, obj);
                }
                this.storeInstanceScope(obj, definition);
                this.getObjectEventTarget().emit(interface_1.ObjectLifeCycleEvent.AFTER_INIT, instance, {
                    context: this.getCurrentMatchedContext(instance),
                    definition,
                });
                initializingSet.delete(definition.id);
            }
        }
        return initializedInstances.get(targetDefinition.id);
    }
    async initializeInstanceAsync(instance, targetDefinition, pendingInitQueue) {
        const initializingSet = new Map();
        const initializedInstances = new Map();
        for (const [obj, definition, initFunc, replaceCallback,] of pendingInitQueue) {
            if (!initializedInstances.has(definition.id)) {
                initializingSet.set(definition.id, obj);
                debug('[core]: load "%s(%s)" from pending init queue and ready to init.', definition.id, definition.name);
                const res = await initFunc();
                if (definition instanceof functionDefinition_1.FunctionDefinition) {
                    initializedInstances.set(definition.id, res);
                    replaceCallback?.(res);
                }
                else {
                    initializedInstances.set(definition.id, obj);
                }
                this.storeInstanceScope(obj, definition);
                this.getObjectEventTarget().emit(interface_1.ObjectLifeCycleEvent.AFTER_INIT, instance, {
                    context: this.getCurrentMatchedContext(instance),
                    definition,
                });
                initializingSet.delete(definition.id);
            }
        }
        return initializedInstances.get(targetDefinition.id);
    }
    storeInstanceScope(instance, definition) {
        if (definition.id) {
            if (definition.isSingletonScope()) {
                debug(`[core]: "${definition.id}(${definition.name})" set to singleton cache`);
                this.singletonCache.set(definition.id, instance);
                this.setInstanceScope(instance, interface_1.ScopeEnum.Singleton);
            }
            else if (definition.isRequestScope()) {
                debug(`[core]: "${definition.id}(${definition.name})" set to register object`);
                this.getCurrentMatchedContext(instance).registerObject(definition.id, instance);
                this.setInstanceScope(instance, interface_1.ScopeEnum.Request);
            }
            else {
                this.setInstanceScope(instance, interface_1.ScopeEnum.Prototype);
            }
        }
    }
    getObjectDefinition(identifier) {
        return this.context.getDefinition(identifier);
    }
    getCurrentMatchedContext(instance) {
        if (instance[constants_1.REQUEST_OBJ_CTX_KEY]) {
            return (instance[constants_1.REQUEST_OBJ_CTX_KEY]?.[constants_1.REQUEST_CTX_UNIQUE_KEY] ?? this.context);
        }
        return this.context;
    }
    translateIdentifiers(ids) {
        return ids.map(id => {
            const definition = this.context.getDefinition(id);
            return definition.path?.name ?? definition.name ?? definition.id;
        });
    }
}
exports.ManagedResolverFactory = ManagedResolverFactory;
//# sourceMappingURL=managedResolverFactory.js.map