"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecoratorManager = void 0;
const constant_1 = require("./constant");
const camelCase_1 = require("../util/camelCase");
const util_1 = require("../util");
const pathFileUtil_1 = require("../util/pathFileUtil");
const metadataManager_1 = require("./metadataManager");
const types_1 = require("../util/types");
const error_1 = require("../error");
const debug = require('util').debuglog('midway:core');
/**
 * This class is used to manage the decorator data of the class
 *
 * @since 3.0.0
 */
class DecoratorManager {
    static saveModule(key, module) {
        if ((0, types_1.isClass)(module)) {
            this.saveProviderId(undefined, module);
        }
        if (!this.moduleStore.has(key)) {
            this.moduleStore.set(key, new Set());
        }
        this.moduleStore.get(key).add(module);
    }
    static listModule(key, filter) {
        const modules = [...(this.moduleStore.get(key) || [])];
        if (filter) {
            return modules.filter(filter);
        }
        else {
            return modules;
        }
    }
    static savePreStartModule(module) {
        this.saveModule(constant_1.PRE_START_MODULE_KEY, module);
    }
    static listPreStartModule() {
        return this.listModule(constant_1.PRE_START_MODULE_KEY);
    }
    static resetModule(key) {
        this.moduleStore.set(key, new Set());
    }
    static clearAllModule() {
        this.moduleStore.clear();
    }
    static saveProviderId(identifier, target) {
        if (this.isProvide(target)) {
            if (identifier) {
                const meta = metadataManager_1.MetadataManager.getOwnMetadata(constant_1.PROVIDE_KEY, target);
                if (meta.id !== identifier) {
                    meta.id = identifier;
                    // save class id and uuid
                    metadataManager_1.MetadataManager.defineMetadata(constant_1.PROVIDE_KEY, meta, target);
                    debug(`update provide: ${target.name} -> ${meta.uuid}`);
                }
            }
        }
        else {
            // save
            const uuid = (0, util_1.generateRandomId)();
            // save class id and uuid
            metadataManager_1.MetadataManager.defineMetadata(constant_1.PROVIDE_KEY, {
                id: identifier,
                originName: target.name,
                uuid,
                name: (0, camelCase_1.camelCase)(target.name),
            }, target);
            debug(`save provide: ${target.name} -> ${uuid}`);
        }
        return target;
    }
    static getProviderId(module) {
        const metaData = metadataManager_1.MetadataManager.getOwnMetadata(constant_1.PROVIDE_KEY, module);
        if (metaData && metaData.id) {
            return metaData.id;
        }
    }
    static getProviderName(module) {
        const metaData = metadataManager_1.MetadataManager.getOwnMetadata(constant_1.PROVIDE_KEY, module);
        if (metaData && metaData.name) {
            return metaData.name;
        }
    }
    static getProviderUUId(module) {
        const metaData = metadataManager_1.MetadataManager.getOwnMetadata(constant_1.PROVIDE_KEY, module);
        if (metaData && metaData.uuid) {
            return metaData.uuid;
        }
    }
    static isProvide(target) {
        return !!metadataManager_1.MetadataManager.getOwnMetadata(constant_1.PROVIDE_KEY, target);
    }
    static createCustomPropertyDecorator(decoratorKey, metadata, implOrOptions = { impl: true }) {
        const implOrOpt = typeof implOrOptions === 'boolean'
            ? {
                impl: implOrOptions,
            }
            : implOrOptions;
        implOrOpt.impl = implOrOpt.impl ?? true;
        return function (target, propertyName) {
            metadataManager_1.MetadataManager.attachMetadata(constant_1.CUSTOM_PROPERTY_INJECT_KEY, {
                propertyName,
                key: decoratorKey,
                metadata,
                options: implOrOptions,
            }, target, propertyName);
        };
    }
    static createCustomMethodDecorator(decoratorKey, metadata, implOrOptions = { impl: true }) {
        const implOrOpt = typeof implOrOptions === 'boolean'
            ? {
                impl: implOrOptions,
            }
            : implOrOptions;
        implOrOpt.impl = implOrOpt.impl ?? true;
        return function (target, propertyName) {
            metadataManager_1.MetadataManager.attachMetadata(constant_1.CUSTOM_METHOD_INJECT_KEY, {
                propertyName,
                key: decoratorKey,
                metadata,
                options: implOrOpt,
            }, target);
        };
    }
    static createCustomParamDecorator(decoratorKey, metadata, implOrOptions = { impl: true }) {
        if (typeof implOrOptions === 'boolean') {
            implOrOptions = { impl: implOrOptions };
        }
        if (implOrOptions.impl === undefined) {
            implOrOptions.impl = true;
        }
        return function (target, propertyName, parameterIndex) {
            metadataManager_1.MetadataManager.attachMetadata(constant_1.CUSTOM_PARAM_INJECT_KEY, {
                key: decoratorKey,
                parameterIndex,
                propertyName,
                metadata,
                options: implOrOptions,
            }, target, propertyName);
        };
    }
}
exports.DecoratorManager = DecoratorManager;
DecoratorManager.moduleStore = new Map();
if (typeof globalThis === 'object' &&
    globalThis['MIDWAY_GLOBAL_DECORATOR_MANAGER']) {
    console.warn('DecoratorManager not singleton and please check @midwayjs/core version by "npm ls @midwayjs/core"');
    const coreModulePathList = (0, pathFileUtil_1.getModuleRequirePathList)('@midwayjs/core');
    if (coreModulePathList.length) {
        console.info('The module may be located in:');
        coreModulePathList.forEach((path, index) => {
            console.info(`${index + 1}. ${path}`);
        });
    }
    throw new error_1.MidwayInconsistentVersionError();
}
else {
    globalThis['MIDWAY_GLOBAL_DECORATOR_MANAGER'] = DecoratorManager;
}
//# sourceMappingURL=decoratorManager.js.map