import 'reflect-metadata';
import { ClassType, TSDesignType } from '../interface';
declare enum ObjectType {
    Class = "class",
    Instance = "instance",
    Object = "object"
}
/**
 * A class that manages metadata for classes and properties
 * This class is a simplified version of the Reflect Metadata API
 * Provides a way to retrieve, define, delete and copy metadata
 *
 * @since 4.0.0
 */
export declare class MetadataManager {
    protected static readonly metadataSymbol: unique symbol;
    protected static readonly metadataClassSymbol: unique symbol;
    protected static readonly metadataPropertySymbol: unique symbol;
    protected static readonly cacheSymbol: unique symbol;
    protected static readonly cleanHooksSymbol: unique symbol;
    protected static readonly isClassSymbol: unique symbol;
    static ObjectType: typeof ObjectType;
    /**
     * A symbol that represents an empty value
     */
    static readonly emptyValueSymbol: unique symbol;
    /**
     * Defines metadata for a target class or property
     * Value will replace the existing metadata
     */
    static defineMetadata(metadataKey: string | symbol, metadataValue: any, target: ClassType | object, propertyKey?: string | symbol): void;
    /**
     * Attaches metadata for a target class or property
     * Value will push to the end of the metadata array and save for own metadata
     */
    static attachMetadata(metadataKey: string | symbol, metadataValue: any, target: ClassType | object, propertyKey?: string | symbol): void;
    /**
     * Retrieves metadata for a target class or property
     */
    static getMetadata<T = any>(metadataKey: string | symbol, target: ClassType | object, propertyKey?: string | symbol): T;
    /**
     * Retrieves own metadata for a target class or property
     */
    static getOwnMetadata<T = any>(metadataKey: string | symbol, target: ClassType | object, propertyKey?: string | symbol): T;
    /**
     * Checks if metadata exists for a target class or property
     */
    static hasMetadata(metadataKey: string | symbol, target: ClassType, propertyKey?: string | symbol): boolean;
    /**
     * Checks if own metadata exists for a target class or property
     */
    static hasOwnMetadata(metadataKey: string | symbol, target: ClassType, propertyKey?: string | symbol): boolean;
    /**
     * Deletes metadata for a target class or property
     */
    static deleteMetadata(metadataKey: string | symbol, target: ClassType, propertyKey?: string | symbol): void;
    /**
     * Get all metadata keys on the entire prototype chain
     * Because we need to get metadata on the entire prototype chain, we do not use cache here, so the performance is poor
     */
    static getMetadataKeys(target: ClassType, propertyKey?: string | symbol): string[];
    /**
     * Get metadata keys on the current class or object
     */
    static getOwnMetadataKeys(target: ClassType, propertyKey?: string | symbol): string[];
    /**
     * Copies metadata from a source class or property to a target class or property
     */
    static copyMetadata(source: ClassType, target: ClassType, options?: {
        metadataFilter?: (metadataKey: string | symbol, propertyKey?: string | symbol) => boolean;
        overwrite?: boolean;
    }): void;
    /**
     * Copies own metadata from a source class or property to a target class or property
     */
    static copyOwnMetadata(source: ClassType, target: ClassType, options?: {
        metadataFilter?: (metadataKey: string | symbol, propertyKey?: string | symbol) => boolean;
        overwrite?: boolean;
    }): void;
    /**
     * Retrieves all properties of the current class that have a specific metadata key and their metadata values.
     *
     * @param metadataKey - The metadata key to check for.
     * @param target - The target class to retrieve properties with metadata from.
     * @returns An object where the key is the property name and the value is the metadata value.
     */
    static getOwnPropertiesWithMetadata<T = Record<string, any>>(metadataKey: string | symbol, target: ClassType): T;
    /**
     * Retrieves all properties of the class and its prototype chain that have a specific metadata key and their metadata values.
     *
     * @param metadataKey - The metadata key to check for.
     * @param target - The target class to retrieve properties with metadata from.
     * @returns An object where the key is the property name and the value is the metadata value.
     */
    static getPropertiesWithMetadata<T = Record<string, any>>(metadataKey: string | symbol, target: ClassType): T;
    /**
     * Gets the type of a property from Reflect metadata
     */
    static getMethodReturnTypes(target: ClassType, propertyKey: string | symbol): any;
    /**
     * Get parameters type by reflect-metadata
     */
    static getMethodParamTypes(target: ClassType, methodName: string | symbol, parameterIndex?: number): any;
    /**
     * Get property(method) type from metadata
     */
    static getPropertyType(target: ClassType, propertyKey: string | symbol): any;
    static transformTypeFromTSDesign(designFn: any): TSDesignType;
    private static copyClassAndPropertyMetadata;
    /**
     * Sets metadata for a target class or property
     */
    private static setMetadata;
    /**
     * Gets or creates the metadata object for a target class or property
     */
    private static getOrCreateMetaObject;
    private static invalidateCache;
    private static setCache;
    private static validCacheConstruct;
    protected static getCache(metadataKey: string | symbol, target: ClassType, propertyKey?: string | symbol): any;
    private static getUnionKey;
    private static formatTarget;
    static ensureTargetType(target: any, type: ObjectType): void;
}
export {};
//# sourceMappingURL=metadataManager.d.ts.map