"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetadataManager = void 0;
require("reflect-metadata");
const types_1 = require("../util/types");
const separator = '\u200A'; // Hair Space
var ObjectType;
(function (ObjectType) {
    ObjectType["Class"] = "class";
    ObjectType["Instance"] = "instance";
    ObjectType["Object"] = "object";
})(ObjectType || (ObjectType = {}));
/**
 * A class that manages metadata for classes and properties
 * This class is a simplified version of the Reflect Metadata API
 * Provides a way to retrieve, define, delete and copy metadata
 *
 * @since 4.0.0
 */
class MetadataManager {
    /**
     * Defines metadata for a target class or property
     * Value will replace the existing metadata
     */
    static defineMetadata(metadataKey, metadataValue, target, propertyKey) {
        target = this.formatTarget(target);
        this.setMetadata(metadataKey, metadataValue, target, propertyKey);
        this.invalidateCache(metadataKey, target, propertyKey);
    }
    /**
     * Attaches metadata for a target class or property
     * Value will push to the end of the metadata array and save for own metadata
     */
    static attachMetadata(metadataKey, metadataValue, target, propertyKey) {
        target = this.formatTarget(target);
        const currentMetadata = this.getOwnMetadata(metadataKey, target, propertyKey);
        if (Array.isArray(currentMetadata)) {
            currentMetadata.push(metadataValue);
            this.defineMetadata(metadataKey, currentMetadata, target, propertyKey);
        }
        else {
            this.defineMetadata(metadataKey, [metadataValue], target, propertyKey);
        }
    }
    /**
     * Retrieves metadata for a target class or property
     */
    static getMetadata(metadataKey, target, propertyKey) {
        target = this.formatTarget(target);
        // find cache first
        const cache = this.getCache(metadataKey, target, propertyKey);
        if (cache === this.emptyValueSymbol) {
            return undefined;
        }
        if (cache !== undefined) {
            return cache;
        }
        // find metadata with prototype chain
        let currentTarget = target;
        while (currentTarget) {
            const metadata = this.getOwnMetadata(metadataKey, currentTarget, propertyKey);
            if (metadata !== undefined) {
                this.setCache(metadataKey, target, currentTarget, metadata, propertyKey);
                return metadata;
            }
            currentTarget = Object.getPrototypeOf(currentTarget);
        }
        this.setCache(metadataKey, target, target, this.emptyValueSymbol, propertyKey);
        return undefined;
    }
    /**
     * Retrieves own metadata for a target class or property
     */
    static getOwnMetadata(metadataKey, target, propertyKey) {
        target = this.formatTarget(target);
        const _metadata = this.getOrCreateMetaObject(target);
        if (propertyKey) {
            return _metadata[this.metadataPropertySymbol][propertyKey]?.[metadataKey];
        }
        else {
            return _metadata[this.metadataClassSymbol][metadataKey];
        }
    }
    /**
     * Checks if metadata exists for a target class or property
     */
    static hasMetadata(metadataKey, target, propertyKey) {
        target = this.formatTarget(target);
        return this.getMetadata(metadataKey, target, propertyKey) !== undefined;
    }
    /**
     * Checks if own metadata exists for a target class or property
     */
    static hasOwnMetadata(metadataKey, target, propertyKey) {
        return this.getOwnMetadata(metadataKey, target, propertyKey) !== undefined;
    }
    /**
     * Deletes metadata for a target class or property
     */
    static deleteMetadata(metadataKey, target, propertyKey) {
        target = this.formatTarget(target);
        const _metadata = this.getOrCreateMetaObject(target);
        if (propertyKey) {
            delete _metadata[this.metadataPropertySymbol][propertyKey]?.[metadataKey];
        }
        else {
            delete _metadata[this.metadataClassSymbol][metadataKey];
        }
        this.invalidateCache(metadataKey, target, propertyKey);
    }
    /**
     * Get all metadata keys on the entire prototype chain
     * Because we need to get metadata on the entire prototype chain, we do not use cache here, so the performance is poor
     */
    static getMetadataKeys(target, propertyKey) {
        target = this.formatTarget(target);
        const keys = new Set();
        let currentTarget = target;
        while (currentTarget) {
            const ownKeys = this.getOwnMetadataKeys(currentTarget, propertyKey);
            ownKeys.forEach(key => keys.add(key));
            currentTarget = Object.getPrototypeOf(currentTarget);
        }
        return Array.from(keys);
    }
    /**
     * Get metadata keys on the current class or object
     */
    static getOwnMetadataKeys(target, propertyKey) {
        target = this.formatTarget(target);
        const _metadata = this.getOrCreateMetaObject(target);
        if (propertyKey) {
            return Object.keys(_metadata[this.metadataPropertySymbol][propertyKey] || {});
        }
        else {
            return Object.keys(_metadata[this.metadataClassSymbol]);
        }
    }
    /**
     * Copies metadata from a source class or property to a target class or property
     */
    static copyMetadata(source, target, options) {
        const { metadataFilter, overwrite = true } = options || {};
        // 合并后的临时存储类级别元数据和属性级别元数据
        const tempClassMetadata = {};
        const tempPropertyMetadata = {};
        // Step 1: 收集原型链上的所有元数据（当前存在的跳过）
        let currentSource = source;
        while (currentSource) {
            const sourceMetadata = this.getOrCreateMetaObject(currentSource);
            // 处理类级别元数据
            const classMetadata = sourceMetadata?.[this.metadataClassSymbol];
            for (const key in classMetadata) {
                if (tempClassMetadata[key] === undefined) {
                    tempClassMetadata[key] = classMetadata[key];
                }
            }
            // 处理属性级别元数据
            const propertyMetadata = sourceMetadata?.[this.metadataPropertySymbol];
            for (const propertyKey in propertyMetadata) {
                if (!tempPropertyMetadata[propertyKey]) {
                    tempPropertyMetadata[propertyKey] = {};
                }
                const propertyMetadataKeys = Object.keys(propertyMetadata[propertyKey]);
                for (const key of propertyMetadataKeys) {
                    if (tempPropertyMetadata[propertyKey][key] === undefined) {
                        tempPropertyMetadata[propertyKey][key] =
                            propertyMetadata[propertyKey][key];
                    }
                }
            }
            currentSource = Object.getPrototypeOf(currentSource);
        }
        // Step 2: 将合并后的元数据应用到目标对象，考虑 options 中的参数
        this.copyClassAndPropertyMetadata(target, metadataFilter, overwrite, tempClassMetadata, tempPropertyMetadata);
    }
    /**
     * Copies own metadata from a source class or property to a target class or property
     */
    static copyOwnMetadata(source, target, options) {
        const { metadataFilter, overwrite = true } = options || {};
        const sourceMetadata = this.getOrCreateMetaObject(source);
        const classMetadata = sourceMetadata?.[this.metadataClassSymbol];
        const propertyMetadata = sourceMetadata?.[this.metadataPropertySymbol];
        this.copyClassAndPropertyMetadata(target, metadataFilter, overwrite, classMetadata, propertyMetadata);
    }
    /**
     * Retrieves all properties of the current class that have a specific metadata key and their metadata values.
     *
     * @param metadataKey - The metadata key to check for.
     * @param target - The target class to retrieve properties with metadata from.
     * @returns An object where the key is the property name and the value is the metadata value.
     */
    static getOwnPropertiesWithMetadata(metadataKey, target) {
        // Ensure the target is a class
        target = this.formatTarget(target);
        // Retrieve or create the metadata object for the target
        const _metadata = this.getOrCreateMetaObject(target);
        // Filter and return the properties that have the specified metadata key and their metadata values
        return Object.keys(_metadata[this.metadataPropertySymbol]).reduce((result, propertyKey) => {
            const metadataValue = _metadata[this.metadataPropertySymbol][propertyKey]?.[metadataKey];
            if (metadataValue !== undefined) {
                result[propertyKey] = metadataValue;
            }
            return result;
        }, {});
    }
    /**
     * Retrieves all properties of the class and its prototype chain that have a specific metadata key and their metadata values.
     *
     * @param metadataKey - The metadata key to check for.
     * @param target - The target class to retrieve properties with metadata from.
     * @returns An object where the key is the property name and the value is the metadata value.
     */
    static getPropertiesWithMetadata(metadataKey, target) {
        const propertiesWithMetadata = {};
        // Traverse the prototype chain
        let currentTarget = this.formatTarget(target);
        while (currentTarget) {
            const _metadata = this.getOrCreateMetaObject(currentTarget);
            Object.keys(_metadata[this.metadataPropertySymbol]).forEach(propertyKey => {
                const metadataValue = _metadata[this.metadataPropertySymbol][propertyKey]?.[metadataKey];
                if (metadataValue !== undefined &&
                    propertiesWithMetadata[propertyKey] === undefined) {
                    propertiesWithMetadata[propertyKey] = metadataValue;
                }
            });
            currentTarget = Object.getPrototypeOf(currentTarget);
        }
        return propertiesWithMetadata;
    }
    /**
     * Gets the type of a property from Reflect metadata
     */
    static getMethodReturnTypes(target, propertyKey) {
        if ((0, types_1.isClass)(target)) {
            target = target.prototype;
        }
        return Reflect.getMetadata('design:returntype', target, propertyKey);
    }
    /**
     * Get parameters type by reflect-metadata
     */
    static getMethodParamTypes(target, methodName, parameterIndex) {
        // 构造器参数必须传递 class，而方法参数需要传递原型对象
        if (methodName && (0, types_1.isClass)(target)) {
            target = target.prototype;
        }
        const types = Reflect.getMetadata('design:paramtypes', target, methodName);
        if (parameterIndex !== undefined && types) {
            return types[parameterIndex];
        }
        return types;
    }
    /**
     * Get property(method) type from metadata
     */
    static getPropertyType(target, propertyKey) {
        if ((0, types_1.isClass)(target)) {
            target = target.prototype;
        }
        return Reflect.getMetadata('design:type', target, propertyKey);
    }
    static transformTypeFromTSDesign(designFn) {
        if ((0, types_1.isNullOrUndefined)(designFn)) {
            return { name: 'undefined', isBaseType: true, originDesign: designFn };
        }
        switch (designFn.name) {
            case 'String':
                return { name: 'string', isBaseType: true, originDesign: designFn };
            case 'Number':
                return { name: 'number', isBaseType: true, originDesign: designFn };
            case 'Boolean':
                return { name: 'boolean', isBaseType: true, originDesign: designFn };
            case 'Symbol':
                return { name: 'symbol', isBaseType: true, originDesign: designFn };
            case 'Object':
                return { name: 'object', isBaseType: true, originDesign: designFn };
            case 'Function':
                return { name: 'function', isBaseType: true, originDesign: designFn };
            default:
                return {
                    name: designFn.name,
                    isBaseType: false,
                    originDesign: designFn,
                };
        }
    }
    static copyClassAndPropertyMetadata(target, metadataFilter, overwrite, classMetadata, propertyMetadata) {
        for (const key in classMetadata) {
            if (!metadataFilter || metadataFilter(key)) {
                const metadataValue = classMetadata[key];
                // 如果不允许覆盖且目标已有元数据，跳过
                if (!overwrite && this.hasOwnMetadata(key, target))
                    continue;
                this.defineMetadata(key, metadataValue, target);
            }
        }
        for (const propertyKey in propertyMetadata) {
            const propertyMetadataKeys = Object.keys(propertyMetadata[propertyKey]);
            for (const key of propertyMetadataKeys) {
                if (!metadataFilter || metadataFilter(key, propertyKey)) {
                    const metadataValue = propertyMetadata[propertyKey][key];
                    // 如果不允许覆盖且目标已有元数据，跳过
                    if (!overwrite && this.hasOwnMetadata(key, target, propertyKey))
                        continue;
                    this.defineMetadata(key, metadataValue, target, propertyKey);
                }
            }
        }
    }
    /**
     * Sets metadata for a target class or property
     */
    static setMetadata(metadataKey, metadataValue, target, propertyKey) {
        const _metadata = this.getOrCreateMetaObject(target);
        const metadataStorage = propertyKey
            ? (_metadata[this.metadataPropertySymbol][propertyKey] ||= {})
            : _metadata[this.metadataClassSymbol];
        metadataStorage[metadataKey] = metadataValue;
    }
    /**
     * Gets or creates the metadata object for a target class or property
     */
    static getOrCreateMetaObject(target) {
        /**
         * metadata construct
         * {
         *  [metadataClassSymbol]: {
         *    [metadataKey]: metadataValue
         *  },
         *  [metadataPropertySymbol]: {
         *    [propertyKey]: {
         *      [metadataKey]: metadataValue
         *    }
         *  }
         */
        // eslint-disable-next-line no-prototype-builtins
        if (!target.hasOwnProperty(this.metadataSymbol)) {
            const _metadata = {
                [this.metadataClassSymbol]: Object.create(null),
                [this.metadataPropertySymbol]: Object.create(null),
            };
            Object.defineProperty(target, this.metadataSymbol, {
                value: _metadata,
                enumerable: false,
                configurable: false,
            });
            return _metadata;
        }
        return target[this.metadataSymbol];
    }
    static invalidateCache(metadataKey, target, propertyKey) {
        // remove current cache
        const unionKey = this.getUnionKey(metadataKey, propertyKey);
        // Remove the specific cache
        // eslint-disable-next-line no-prototype-builtins
        if (target.hasOwnProperty(this.cacheSymbol)) {
            delete target[this.cacheSymbol]?.[unionKey];
        }
        // Execute hooks, passing the unionKey to clear
        const cleanHooks = target[this.cleanHooksSymbol];
        if (cleanHooks) {
            target[this.cleanHooksSymbol] = cleanHooks.filter(hook => !hook(unionKey));
        }
    }
    static setCache(metadataKey, target, protoTarget, value, propertyKey) {
        this.validCacheConstruct(target);
        const unionKey = this.getUnionKey(metadataKey, propertyKey);
        if (target !== protoTarget) {
            this.validCacheConstruct(protoTarget);
            // Register a clean hook to the prototype target and clean the cache when the prototype target value is changed
            protoTarget[this.cleanHooksSymbol].push((keyToClear) => {
                // Only delete cache if the key matches
                if (keyToClear === unionKey) {
                    delete target[this.cacheSymbol]?.[unionKey];
                    // Indicates that this hook can be removed
                    return true;
                }
                return false;
            });
        }
        target[this.cacheSymbol][unionKey] = value;
    }
    static validCacheConstruct(target) {
        // eslint-disable-next-line no-prototype-builtins
        if (!target.hasOwnProperty(this.cacheSymbol)) {
            Object.defineProperty(target, this.cacheSymbol, {
                value: Object.create(null),
                enumerable: false,
                configurable: false,
            });
        }
        // eslint-disable-next-line no-prototype-builtins
        if (!target.hasOwnProperty(this.cleanHooksSymbol)) {
            Object.defineProperty(target, this.cleanHooksSymbol, {
                value: [],
                enumerable: false,
                configurable: false,
                writable: true,
            });
        }
    }
    static getCache(metadataKey, target, propertyKey) {
        // eslint-disable-next-line no-prototype-builtins
        if (target.hasOwnProperty(this.cacheSymbol)) {
            return target[this.cacheSymbol]?.[this.getUnionKey(metadataKey, propertyKey)];
        }
    }
    static getUnionKey(metadataKey, propertyKey) {
        return propertyKey
            ? `${propertyKey.toString()}${separator}${metadataKey.toString()}`
            : metadataKey.toString();
    }
    static formatTarget(target) {
        // eslint-disable-next-line no-prototype-builtins
        let ret = target.hasOwnProperty(this.isClassSymbol)
            ? target[this.isClassSymbol]
            : undefined;
        if (!ret) {
            const isClassRet = (0, types_1.isClass)(target);
            if (isClassRet) {
                ret = ObjectType.Class;
            }
            else if ((0, types_1.isClass)(target.constructor)) {
                ret = ObjectType.Instance;
            }
            else {
                ret = ObjectType.Object;
            }
            Object.defineProperty(target, this.isClassSymbol, {
                value: ret,
                enumerable: false,
                configurable: false,
            });
        }
        return ret === ObjectType.Instance ? target.constructor : target;
    }
    static ensureTargetType(target, type) {
        // eslint-disable-next-line no-prototype-builtins
        const ret = target.hasOwnProperty(this.isClassSymbol)
            ? target[this.isClassSymbol]
            : undefined;
        if (!ret) {
            Object.defineProperty(target, this.isClassSymbol, {
                value: type,
                enumerable: false,
                configurable: false,
            });
        }
    }
}
exports.MetadataManager = MetadataManager;
MetadataManager.metadataSymbol = Symbol.for('midway.metadata');
MetadataManager.metadataClassSymbol = Symbol.for('midway.metadata.class');
MetadataManager.metadataPropertySymbol = Symbol.for('midway.metadata.property');
MetadataManager.cacheSymbol = Symbol.for('midway.metadata.cache');
MetadataManager.cleanHooksSymbol = Symbol.for('midway.clean.hooks');
MetadataManager.isClassSymbol = Symbol.for('midway.metadata.isClass');
MetadataManager.ObjectType = ObjectType;
/**
 * A symbol that represents an empty value
 */
MetadataManager.emptyValueSymbol = Symbol.for('midway.metadata.empty');
//# sourceMappingURL=metadataManager.js.map