"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareGlobalApplicationContext = exports.prepareGlobalApplicationContextAsync = exports.destroyGlobalApplicationContext = exports.initializeGlobalApplicationContext = void 0;
const _1 = require("./");
const config_default_1 = require("./config/config.default");
const util = require("util");
const slsFunctionService_1 = require("./service/slsFunctionService");
const healthService_1 = require("./service/healthService");
const componentLoader_1 = require("./context/componentLoader");
const util_1 = require("./util");
const asyncContextManager_1 = require("./common/asyncContextManager");
const performanceManager_1 = require("./common/performanceManager");
const debug = util.debuglog('midway:debug');
let stepIdx = 1;
function printStepDebugInfo(stepInfo) {
    debug(`\n\nStep ${stepIdx++}: ${stepInfo}\n`);
}
/**
 * midway framework main entry, this method bootstrap all service and framework.
 * @param globalOptions
 */
async function initializeGlobalApplicationContext(globalOptions) {
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.INITIALIZE);
    const applicationContext = await prepareGlobalApplicationContextAsync(globalOptions);
    printStepDebugInfo('Init logger');
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.LOGGER_PREPARE);
    // init logger
    const loggerService = await applicationContext.getAsync(_1.MidwayLoggerService, [
        applicationContext,
        globalOptions,
    ]);
    if (loggerService.getLogger('appLogger')) {
        // register global logger
        applicationContext.registerObject('logger', loggerService.getLogger('appLogger'));
    }
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.LOGGER_PREPARE);
    printStepDebugInfo('Init MidwayMockService');
    // mock support
    await applicationContext.getAsync(_1.MidwayMockService, [applicationContext]);
    printStepDebugInfo('Init framework');
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.FRAMEWORK_PREPARE);
    // framework/config/plugin/logger/app decorator support
    await applicationContext.getAsync(_1.MidwayFrameworkService, [
        applicationContext,
        globalOptions,
    ]);
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.FRAMEWORK_PREPARE);
    printStepDebugInfo('Init lifecycle');
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.LIFECYCLE_PREPARE);
    // lifecycle support
    await applicationContext.getAsync(_1.MidwayLifeCycleService, [
        applicationContext,
    ]);
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.LIFECYCLE_PREPARE);
    printStepDebugInfo('Init pre-start modules');
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.PRELOAD_MODULE_PREPARE);
    // some pre-start module init
    const modules = _1.DecoratorManager.listPreStartModule();
    for (const module of modules) {
        // pre-start init context
        await applicationContext.getAsync(module);
    }
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.PRELOAD_MODULE_PREPARE);
    printStepDebugInfo('End of initialize and start');
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.INITIALIZE);
    return applicationContext;
}
exports.initializeGlobalApplicationContext = initializeGlobalApplicationContext;
async function destroyGlobalApplicationContext(applicationContext) {
    const loggerService = await applicationContext.getAsync(_1.MidwayLoggerService);
    const loggerFactory = loggerService.getCurrentLoggerFactory();
    // stop lifecycle
    const lifecycleService = await applicationContext.getAsync(_1.MidwayLifeCycleService);
    await lifecycleService.stop();
    // stop container
    await applicationContext.stop();
    loggerFactory.close();
    performanceManager_1.MidwayPerformanceManager.cleanAll();
    global['MIDWAY_APPLICATION_CONTEXT'] = undefined;
    global['MIDWAY_MAIN_FRAMEWORK'] = undefined;
    // reset counter
    stepIdx = 1;
}
exports.destroyGlobalApplicationContext = destroyGlobalApplicationContext;
/**
 * prepare applicationContext
 * @param globalOptions
 */
async function prepareGlobalApplicationContextAsync(globalOptions) {
    printStepDebugInfo('Ready to create applicationContext');
    debug('[core]: start "initializeGlobalApplicationContext"');
    debug(`[core]: bootstrap options = ${util.inspect(globalOptions)}`);
    const appDir = globalOptions.appDir ?? '';
    const baseDir = globalOptions.baseDir ?? '';
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.METADATA_PREPARE);
    // new container
    const applicationContext = globalOptions.applicationContext ?? new _1.MidwayContainer();
    global['MIDWAY_APPLICATION_CONTEXT'] = applicationContext;
    // register baseDir and appDir
    applicationContext.registerObject('baseDir', baseDir);
    applicationContext.registerObject('appDir', appDir);
    if (!globalOptions.asyncContextManager) {
        globalOptions.asyncContextManager = new asyncContextManager_1.AsyncLocalStorageContextManager();
    }
    debug('[core]: set default module load type and entry file');
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.METADATA_PREPARE);
    debug('[core]: set default file detector');
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.DETECTOR_PREPARE);
    printStepDebugInfo('Ready module detector');
    if (!globalOptions.moduleLoadType) {
        globalOptions.moduleLoadType = 'commonjs';
    }
    // set entry file
    globalOptions.imports = [
        ...(globalOptions.imports ?? []),
        await (0, util_1.findProjectEntryFile)(appDir, baseDir, globalOptions.moduleLoadType),
    ];
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.DETECTOR_PREPARE);
    printStepDebugInfo('Binding built-in service');
    // bind built-in service
    applicationContext.bindClass(_1.MidwayEnvironmentService);
    applicationContext.bindClass(_1.MidwayInformationService);
    applicationContext.bindClass(_1.MidwayAspectService);
    applicationContext.bindClass(_1.MidwayDecoratorService);
    applicationContext.bindClass(_1.MidwayConfigService);
    applicationContext.bindClass(_1.MidwayLoggerService);
    applicationContext.bindClass(_1.MidwayApplicationManager);
    applicationContext.bindClass(_1.MidwayFrameworkService);
    applicationContext.bindClass(_1.MidwayMiddlewareService);
    applicationContext.bindClass(_1.MidwayLifeCycleService);
    applicationContext.bindClass(_1.MidwayMockService);
    applicationContext.bindClass(_1.MidwayWebRouterService);
    applicationContext.bindClass(slsFunctionService_1.MidwayServerlessFunctionService);
    applicationContext.bindClass(healthService_1.MidwayHealthService);
    applicationContext.bindClass(_1.MidwayPriorityManager);
    printStepDebugInfo('Init MidwayConfigService, MidwayAspectService and MidwayDecoratorService');
    // init default environment
    const environmentService = applicationContext.get(_1.MidwayEnvironmentService);
    environmentService.setModuleLoadType(globalOptions.moduleLoadType);
    // init default config
    const configService = applicationContext.get(_1.MidwayConfigService);
    configService.add([
        {
            default: config_default_1.default,
        },
    ]);
    // init aop support
    applicationContext.get(_1.MidwayAspectService, [applicationContext]);
    // init decorator service
    applicationContext.get(_1.MidwayDecoratorService, [applicationContext]);
    printStepDebugInfo('Binding preload module');
    // bind preload module
    if (globalOptions.preloadModules && globalOptions.preloadModules.length) {
        for (const preloadModule of globalOptions.preloadModules) {
            applicationContext.bindClass(preloadModule);
        }
    }
    printStepDebugInfo('Load imports(component) and user code configuration module');
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.DEFINITION_PREPARE);
    // load configuration
    const componentConfigurationLoader = new componentLoader_1.ComponentConfigurationLoader(applicationContext);
    const importModules = [...(globalOptions.imports ?? [])];
    for (const mod of importModules) {
        if (mod) {
            await componentConfigurationLoader.load(mod);
        }
    }
    for (const ns of componentConfigurationLoader.getNamespaceList()) {
        applicationContext.addNamespace(ns);
        debug(`[core]: load configuration in namespace="${ns}" complete`);
    }
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.DEFINITION_PREPARE);
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.CONFIG_LOAD);
    if (globalOptions.globalConfig) {
        if (Array.isArray(globalOptions.globalConfig)) {
            configService.add(globalOptions.globalConfig);
        }
        else {
            configService.addObject(globalOptions.globalConfig);
        }
    }
    printStepDebugInfo('Load config file');
    // merge config
    configService.load();
    debug('[core]: Current config = %j', configService.getConfiguration());
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.CONFIG_LOAD);
    // middleware support
    applicationContext.get(_1.MidwayMiddlewareService, [applicationContext]);
    return applicationContext;
}
exports.prepareGlobalApplicationContextAsync = prepareGlobalApplicationContextAsync;
/**
 * prepare applicationContext, it use in egg framework, hooks and serverless function generator
 * @param globalOptions
 */
function prepareGlobalApplicationContext(globalOptions) {
    printStepDebugInfo('Ready to create applicationContext');
    debug('[core]: start "initializeGlobalApplicationContext"');
    debug(`[core]: bootstrap options = ${util.inspect(globalOptions)}`);
    const appDir = globalOptions.appDir ?? '';
    const baseDir = globalOptions.baseDir ?? '';
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.METADATA_PREPARE);
    // new container
    const applicationContext = globalOptions.applicationContext ?? new _1.MidwayContainer();
    global['MIDWAY_APPLICATION_CONTEXT'] = applicationContext;
    // register baseDir and appDir
    applicationContext.registerObject('baseDir', baseDir);
    applicationContext.registerObject('appDir', appDir);
    if (!globalOptions.asyncContextManager) {
        globalOptions.asyncContextManager = new asyncContextManager_1.AsyncLocalStorageContextManager();
    }
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.METADATA_PREPARE);
    debug('[core]: set default module load type and entry file');
    if (!globalOptions.moduleLoadType) {
        globalOptions.moduleLoadType = 'commonjs';
    }
    // set entry file
    globalOptions.imports = [
        ...(globalOptions.imports ?? []),
        (0, util_1.findProjectEntryFileSync)(appDir, baseDir),
    ];
    printStepDebugInfo('Binding built-in service');
    // bind built-in service
    applicationContext.bindClass(_1.MidwayEnvironmentService);
    applicationContext.bindClass(_1.MidwayInformationService);
    applicationContext.bindClass(_1.MidwayAspectService);
    applicationContext.bindClass(_1.MidwayDecoratorService);
    applicationContext.bindClass(_1.MidwayConfigService);
    applicationContext.bindClass(_1.MidwayLoggerService);
    applicationContext.bindClass(_1.MidwayApplicationManager);
    applicationContext.bindClass(_1.MidwayFrameworkService);
    applicationContext.bindClass(_1.MidwayMiddlewareService);
    applicationContext.bindClass(_1.MidwayLifeCycleService);
    applicationContext.bindClass(_1.MidwayMockService);
    applicationContext.bindClass(_1.MidwayWebRouterService);
    applicationContext.bindClass(slsFunctionService_1.MidwayServerlessFunctionService);
    applicationContext.bindClass(healthService_1.MidwayHealthService);
    applicationContext.bindClass(_1.MidwayPriorityManager);
    printStepDebugInfo('Init MidwayConfigService, MidwayAspectService and MidwayDecoratorService');
    // init default environment
    const environmentService = applicationContext.get(_1.MidwayEnvironmentService);
    environmentService.setModuleLoadType(globalOptions.moduleLoadType);
    // init default config
    const configService = applicationContext.get(_1.MidwayConfigService);
    configService.add([
        {
            default: config_default_1.default,
        },
    ]);
    // init aop support
    applicationContext.get(_1.MidwayAspectService, [applicationContext]);
    // init decorator service
    applicationContext.get(_1.MidwayDecoratorService, [applicationContext]);
    printStepDebugInfo('Binding preload module');
    // bind preload module
    if (globalOptions.preloadModules && globalOptions.preloadModules.length) {
        for (const preloadModule of globalOptions.preloadModules) {
            applicationContext.bindClass(preloadModule);
        }
    }
    printStepDebugInfo('Load imports(component) and user code configuration module');
    // load configuration
    const componentConfigurationLoader = new componentLoader_1.ComponentConfigurationLoader(applicationContext);
    const importModules = [...(globalOptions.imports ?? [])];
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.DEFINITION_PREPARE);
    for (const mod of importModules) {
        if (mod) {
            componentConfigurationLoader.loadSync(mod);
        }
    }
    for (const ns of componentConfigurationLoader.getNamespaceList()) {
        applicationContext.addNamespace(ns);
        debug(`[core]: load configuration in namespace="${ns}" complete`);
    }
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.DEFINITION_PREPARE);
    performanceManager_1.MidwayInitializerPerformanceManager.markStart(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.CONFIG_LOAD);
    if (globalOptions.globalConfig) {
        if (Array.isArray(globalOptions.globalConfig)) {
            configService.add(globalOptions.globalConfig);
        }
        else {
            configService.addObject(globalOptions.globalConfig);
        }
    }
    printStepDebugInfo('Load config file');
    // merge config
    configService.load();
    debug('[core]: Current config = %j', configService.getConfiguration());
    performanceManager_1.MidwayInitializerPerformanceManager.markEnd(performanceManager_1.MidwayInitializerPerformanceManager.MEASURE_KEYS.CONFIG_LOAD);
    // middleware support
    applicationContext.get(_1.MidwayMiddlewareService, [applicationContext]);
    return applicationContext;
}
exports.prepareGlobalApplicationContext = prepareGlobalApplicationContext;
//# sourceMappingURL=setup.js.map